(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/demo.js                                                      //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
allowDemo = Meteor.settings && Meteor.settings["public"] && Meteor.settings["public"].allowDemoAccounts;
                                                                       //
var DEMO_EXPIRATION_MS = 60 * 60 * 1000;                               // 20
var DEMO_GRACE_MS = 10 * 60 * 1000; // time between expiration and deletion
                                                                       //
if (Meteor.isServer) {                                                 // 23
  Accounts.validateLoginAttempt(function (attempt) {                   // 24
    // Enforce expiration times for demo accounts.                     //
                                                                       //
    if (attempt.user && attempt.user.expires) {                        // 27
      var expireIn = attempt.user.expires.getTime() - Date.now() + DEMO_GRACE_MS;
      if (expireIn < 0) {                                              // 29
        throw new Meteor.Error(403, "This demo account has expired.");
      }                                                                //
                                                                       //
      if (!allowDemo) {                                                // 33
        // An expiration time implies that this is a demo account, but those have been disabled.
        // Perhaps the config was just changed.                        //
        throw new Meteor.Error(403, "Demo accounts have been disabled.");
      }                                                                //
                                                                       //
      // Force connection close when account expires, so that the client reconnects and
      // re-authenticates, which fails.                                //
      var connection = attempt.connection;                             // 41
      var handle = Meteor.setTimeout(function () {                     // 42
        connection.close();                                            // 42
      }, expireIn);                                                    //
      connection.onClose(function () {                                 // 43
        Meteor.clearTimeout(handle);                                   // 43
      });                                                              //
    }                                                                  //
                                                                       //
    return true;                                                       // 46
  });                                                                  //
                                                                       //
  function cleanupExpiredUsers() {                                     // 49
    // Delete expired demo accounts and all their grains.              //
                                                                       //
    var now = new Date(Date.now() - DEMO_GRACE_MS);                    // 52
    Meteor.users.find({ expires: { $lt: now } }, { fields: { _id: 1, loginIdentities: 1, lastActive: 1, appDemoId: 1 } }).forEach(function (user) {
      Grains.find({ userId: user._id }, { fields: { _id: 1, lastUsed: 1, appId: 1 } }).forEach(function (grain) {
        console.log("delete grain: " + grain._id);                     // 58
        ApiTokens.remove({ grainId: grain._id });                      // 59
        Grains.remove(grain._id);                                      // 60
        if (grain.lastUsed) {                                          // 61
          DeleteStats.insert({ type: "demoGrain", lastActive: grain.lastUsed, appId: grain.appId });
        }                                                              //
        globalBackend.deleteGrain(grain._id, user._id);                // 64
      });                                                              //
      console.log("delete user: " + user._id);                         // 66
      // We intentionally do not do `ApiTokens.remove({accountId: user._id})`, because some such
      // tokens might still play an active role in the sharing graph.  //
      Contacts.remove({ ownerId: user._id });                          // 69
      UserActions.remove({ userId: user._id });                        // 70
      Notifications.remove({ userId: user._id });                      // 71
      Meteor.users.remove(user._id);                                   // 72
      waitPromise(globalBackend.cap().deleteUser(user._id));           // 73
      if (user.loginIdentities && user.lastActive) {                   // 74
        // When deleting a user, we can specify it as a "normal" user  //
        // (type: user) or as a user who started out by using the app  //
        // demo feature (type: appDemoUser).                           //
        var deleteStatsType = "demoUser";                              // 78
        var isAppDemoUser = !!user.appDemoId;                          // 79
        if (isAppDemoUser) {                                           // 80
          deleteStatsType = "appDemoUser";                             // 81
        }                                                              //
                                                                       //
        // Intentionally record deleted users at time of deletion to avoid miscounting users that
        // were demoing just before the day rolled over.               //
        DeleteStats.insert({ type: deleteStatsType, lastActive: new Date(), appId: user.appDemoId });
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  if (allowDemo) {                                                     // 91
    Meteor.methods({                                                   // 92
      createDemoUser: function (displayName, appDemoId) {              // 93
        // This is a login method that creates a new temporary user    //
        // every time it is used.                                      //
        //                                                             //
        // appDemoId is important for stats; see cleanupExpiredUsers().
        check(displayName, String);                                    // 98
        check(appDemoId, Match.OneOf(undefined, null, String));        // 99
                                                                       //
        // Create the new user.                                        //
        var newUser = { expires: new Date(Date.now() + DEMO_EXPIRATION_MS) };
        if (appDemoId) {                                               // 103
          newUser.appDemoId = appDemoId;                               // 104
        }                                                              //
        var userId = Accounts.insertUserDoc({ profile: { name: displayName } }, newUser);
                                                                       //
        // Log them in on this connection.                             //
        return Accounts._loginMethod(this, "createDemoUser", arguments, "demo", function () {
          return { userId: userId };                                   // 110
        });                                                            //
      },                                                               //
                                                                       //
      testExpireDemo: function () {                                    // 113
        if (!isDemoUser()) throw new Meteor.Error(403, "not a demo user");
                                                                       //
        var newExpires = new Date(Date.now() + 15000);                 // 116
        if (Meteor.user().expires.getTime() < newExpires.getTime()) {  // 117
          throw new Meteor.Error(403, "can't exend demo");             // 118
        }                                                              //
        Meteor.users.update(this.userId, { $set: { expires: newExpires } });
      }                                                                //
    });                                                                //
                                                                       //
    // If demo mode is enabled, we permit the client to subscribe to   //
    // information about an app by appId. If this were available in    //
    // non-demo mode, then anonymous users could effectively ask the   //
    // server which apps are installed.                                //
    Meteor.publish("appInfo", function (appId) {                       // 128
      // This publishes info about an app, including the latest        //
      // version of it. Once you log in, it also publishes your        //
      // list of UserActions.                                          //
      check(appId, String);                                            // 132
                                                                       //
      var packageCursor = Packages.find({ appId: appId }, { sort: { "manifest.appVersion": -1 } });
                                                                       //
      var pkg = packageCursor.fetch()[0];                              // 138
                                                                       //
      // This allows us to avoid creating duplicate UserActions.       //
      if (this.userId) {                                               // 141
        return [packageCursor, UserActions.find({ userId: this.userId, appId: appId })];
      }                                                                //
                                                                       //
      return packageCursor;                                            // 148
    });                                                                //
                                                                       //
    SandstormDb.periodicCleanup(DEMO_EXPIRATION_MS, cleanupExpiredUsers);
  } else {                                                             //
    // Just run once, in case the config just changed from allowing demos to prohibiting them.
    Meteor.setTimeout(cleanupExpiredUsers, DEMO_EXPIRATION_MS + DEMO_GRACE_MS);
  }                                                                    //
}                                                                      //
                                                                       //
if (Meteor.isClient && allowDemo) {                                    // 158
  Meteor.loginWithDemo = function (options, callback) {                // 159
    Router.go("demo");                                                 // 160
    callback();                                                        // 161
  };                                                                   //
  // Note: We intentionally don't register the demo service with Accounts.registerService(); we
  //   don't want it to appear in the sign-in drop-down.               //
                                                                       //
  window.testExpireDemo = function () {                                // 166
    Meteor.call("testExpireDemo");                                     // 167
  };                                                                   //
                                                                       //
  Template.demo.events({                                               // 170
    "click button.start": function (event) {                           // 171
      var displayName = "Demo User";                                   // 172
                                                                       //
      var userCallbackFunction = function (err) {                      // 174
        if (err) {                                                     // 175
          window.alert(err);                                           // 176
        } else {                                                       //
          Router.go("root");                                           // 178
        }                                                              //
      };                                                               //
                                                                       //
      if (isSignedUpOrDemo()) {                                        // 182
        userCallbackFunction();                                        // 183
      } else {                                                         //
        Accounts.callLoginMethod({                                     // 185
          methodName: "createDemoUser",                                // 186
          methodArguments: ["Demo User", null],                        // 187
          userCallback: userCallbackFunction                           // 188
        });                                                            //
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.appdemo.events({                                            // 194
    "click button.start": function (event) {                           // 195
      // When clicking on the createDemoUser button on the app demo,   //
      // we want to:                                                   //
      //                                                               //
      // 1. Create the Demo User if they are not logged in.            //
      //                                                               //
      // 2. Log the user in as this Demo User if we created it.        //
      //                                                               //
      // 3. Install the chosen app.                                    //
      //                                                               //
      // 4. Create a new grain with this app.                          //
      //                                                               //
      // 5. Take them into this grain.                                 //
                                                                       //
      // calculate the appId                                           //
                                                                       //
      // We copy the appId into the scope so the autorun function can access it.
      var appId = this.appId;                                          // 212
      var signingIn = false;                                           // 213
                                                                       //
      // TODO(cleanup): Is there a better way to do this? Before multi-identity, we could use the
      //   `userCallback` option of `Accounts.callLoginMethod()`, but now that doesn't work because
      //   it fires too early.                                         //
      //                                                               //
      // Note that we don't use Template.instance().autorun() because the template gets destroyed
      // and recreated during account creation.                        //
      var done = false;                                                // 221
      var handle = Tracker.autorun(function () {                       // 222
        if (done) return;                                              // 223
                                                                       //
        if (!isSignedUpOrDemo()) {                                     // 225
          if (!signingIn) {                                            // 226
            signingIn = true;                                          // 227
            // 1. Create the Demo User & 2. Log the user in as this Demo User.
            Accounts.callLoginMethod({                                 // 229
              methodName: "createDemoUser",                            // 230
              methodArguments: ["Demo User", appId]                    // 231
            });                                                        //
          } else {                                                     //
            return;                                                    // 234
          }                                                            //
        } else {                                                       //
          // `handle` may not have been assigned yet if this is the first run of the callback.
          // But we definitely don't want the callback to run again. So we have to resort to
          // setting a flag that disables subsequent runs, and then deferring the actual stop. Ick.
          done = true;                                                 // 240
          Meteor.defer(function () {                                   // 241
            handle.stop();                                             // 241
          });                                                          //
                                                                       //
          // First, find the package ID, since that is what            //
          // addUserActions takes. Choose the package ID with          //
          // highest version number.                                   //
          var packageId = Packages.findOne({ appId: appId }, { sort: { "manifest.appVersion": -1 } })._id;
                                                                       //
          // 3. Install this app for the user, if needed.              //
          if (UserActions.find({ appId: appId, userId: Meteor.userId() }).count() == 0) {
            globalDb.addUserActions(packageId);                        // 251
          }                                                            //
                                                                       //
          // 4. Create new grain and 5. browse to it.                  //
          launchAndEnterGrainByPackageId(packageId);                   // 255
        }                                                              //
      });                                                              //
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
Router.map(function () {                                               // 262
  this.route("demo", {                                                 // 263
    path: "/demo",                                                     // 264
    waitOn: function () {                                              // 265
      return Meteor.subscribe("credentials");                          // 266
    },                                                                 //
    data: function () {                                                // 268
      return {                                                         // 269
        allowDemo: allowDemo,                                          // 270
        pageTitle: "Demo",                                             // 271
        isDemoUser: isDemoUser()                                       // 272
      };                                                               //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("demoRestart", {                                          // 277
    path: "/demo-restart",                                             // 278
    waitOn: function () {                                              // 279
      return Meteor.subscribe("credentials");                          // 280
    },                                                                 //
    data: function () {                                                // 282
      Meteor.logout();                                                 // 283
      Router.go("demo");                                               // 284
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 289
  this.route("appdemo", {                                              // 290
    path: "/appdemo/:appId",                                           // 291
    waitOn: function () {                                              // 292
      return Meteor.subscribe("appInfo", this.params.appId);           // 293
    },                                                                 //
    data: function () {                                                // 295
      // find the newest (highest version, so "first" when sorting by  //
      // inverse order) matching package.                              //
      var thisPackage = Packages.findOne({ appId: this.params.appId }, { sort: { "manifest.appVersion": -1 } });
                                                                       //
      // In the case that the app requested is not present, we show    //
      // this string as the app name.                                  //
      var appName = 'missing package';                                 // 303
                                                                       //
      if (thisPackage) {                                               // 305
        appName = SandstormDb.appNameFromPackage(thisPackage);         // 306
      }                                                                //
                                                                       //
      return {                                                         // 309
        allowDemo: allowDemo,                                          // 310
        // For appdemo, we always allow you to start the demo, because
        // the this refers to the app demo, and if a visitor clicks    //
        // visits /appdemo/:appId once and creates a Demo User         //
        // account, and then clicks a different /appdemo/:appId URL to
        // demo a different app, we want them to experience the joy of
        // trying the second app.                                      //
        shouldShowStartDemo: true,                                     // 317
        appName: appName,                                              // 318
        pageTitle: appName + " Demo on Sandstorm",                     // 319
        appId: this.params.appId,                                      // 320
        isDemoUser: isDemoUser()                                       // 321
      };                                                               //
    }                                                                  //
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=demo.js.map
