(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/devAccounts.js                                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var allowDevAccounts = Meteor.settings && Meteor.settings["public"] && Meteor.settings["public"].allowDevAccounts;
                                                                       //
Accounts.identityServices.dev = {                                      // 20
  isEnabled: function () {                                             // 21
    return allowDevAccounts;                                           // 22
  },                                                                   //
  loginTemplate: {                                                     // 24
    name: "devLoginForm",                                              // 25
    priority: -10 }                                                    // 26
};                                                                     //
                                                                       //
// Put it at the top.                                                  //
if (allowDevAccounts) {                                                // 30
  if (Meteor.isServer) {                                               // 31
    var Crypto = Npm.require("crypto");                                // 32
                                                                       //
    Meteor.methods({                                                   // 34
      createDevAccount: function (displayName, isAdmin, profile, unverifiedEmail) {
        // This is a login method that creates or logs in a dev account with the given displayName
                                                                       //
        check(displayName, String);                                    // 38
        check(isAdmin, Match.OneOf(undefined, null, Boolean));         // 39
        check(profile, Match.OneOf(undefined, null, Object));          // 40
        check(unverifiedEmail, Match.OneOf(undefined, null, String));  // 41
                                                                       //
        isAdmin = isAdmin || false;                                    // 43
                                                                       //
        profile = profile || {};                                       // 45
        profile.name = profile.name || displayName;                    // 46
        var hasCompletedSignup = !!unverifiedEmail && !!profile.pronoun && !!profile.handle;
                                                                       //
        var user = Meteor.users.findOne({ "services.dev.name": displayName });
        var userId;                                                    // 50
                                                                       //
        if (user) {                                                    // 52
          userId = user._id;                                           // 53
        } else {                                                       //
          userId = Accounts.insertUserDoc({ profile: profile,          // 55
            unverifiedEmail: unverifiedEmail }, { services: { dev: { name: displayName,
                isAdmin: isAdmin,                                      // 59
                hasCompletedSignup: hasCompletedSignup } } });         // 60
        }                                                              //
        // Log them in on this connection.                             //
        return Accounts._loginMethod(this, "createDevAccount", arguments, "dev", function () {
          return { userId: userId };                                   // 64
        });                                                            //
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  if (Meteor.isClient) {                                               // 69
    loginDevAccount = function (displayName, isAdmin) {                // 70
      Accounts.callLoginMethod({                                       // 71
        methodName: "createDevAccount",                                // 72
        methodArguments: [displayName, isAdmin],                       // 73
        userCallback: function (err) {                                 // 74
          if (err) {                                                   // 75
            window.alert(err);                                         // 76
          }                                                            //
        }                                                              //
      });                                                              //
    };                                                                 //
                                                                       //
    loginDevAccountFast = function (displayName, isAdmin) {            // 82
      return new Promise(function (resolve, reject) {                  // 83
        // This skips the firstSignUp page. Mostly used for testing purposes.
        var profile = {                                                // 85
          name: displayName,                                           // 86
          pronoun: "robot",                                            // 87
          handle: "_" + displayName.toLowerCase()                      // 88
        };                                                             //
                                                                       //
        Accounts.callLoginMethod({                                     // 91
          methodName: "createDevAccount",                              // 92
          methodArguments: [displayName, isAdmin, profile, displayName + "@example.com"],
          userCallback: function (err) {                               // 94
            if (err) {                                                 // 95
              reject(new Error(err));                                  // 96
            } else {                                                   //
              Router.go("apps");                                       // 98
              resolve();                                               // 99
            }                                                          //
          }                                                            //
        });                                                            //
      });                                                              //
    };                                                                 //
  }                                                                    //
}                                                                      //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=devAccounts.js.map
